import { routerRedux } from 'dva/router';
import { message } from 'antd';
import {
  createResource,
  deleteResource,
  updateResource,
  queryImageResource,
  queryVideoResource,
} from '../services/resource';

export default {
  namespace: 'resource',

  state: {
    list: [],
    pageNo: 1,
    pageSize: 15,
    totalSize: 0,
  },

  effects: {
    *fetchImageList({ payload }, { call, put }) {
      const response = yield call(queryImageResource, payload);
      if (response.success) {
        yield put({
          type: 'querySuccess',
          payload: {
            list: response.data.list || [],
            pageSize: response.data.pageSize,
            totalSize: response.data.totalSize,
            pageNo: response.data.pageNo,
          },
        });
      }
    },
    *fetchVideoList({ payload }, { call, put }) {
      const response = yield call(queryVideoResource, payload);
      if (response.success) {
        yield put({
          type: 'querySuccess',
          payload: {
            list: response.data.list || [],
            pageSize: response.data.pageSize,
            totalSize: response.data.totalSize,
            pageNo: response.data.pageNo,
          },
        });
      }
    },
    *createImage({ payload }, { call, put }) {
      const response = yield call(createResource, payload);
      if (response && response.success) {
        yield put(
          routerRedux.push({
            pathname: '/resource/picture-upload/result',
            state: response.data,
          })
        );
      }
    },
    *createVideo({ payload, states }, { call, put }) {
      const response = yield call(createResource, payload);
      if (response && response.success) {
        yield put(
          routerRedux.push({
            pathname: '/resource/video',
            state: states,
          })
        );
      }
    },
    *updateImage({ payload, states }, { call, put }) {
      const response = yield call(updateResource, payload);
      if (response && response.success) {
        message.success('修改图片成功');
        yield put(routerRedux.push({
          pathname: '/resource/picture',
          state: states,
        }));
      }
    },
    *updateVideo({ payload, states }, { call, put }) {
      const response = yield call(updateResource, payload);
      if (response && response.success) {
        message.success('修改视频成功');
        yield put(routerRedux.push({
          pathname: '/resource/video',
          state: states,
        }));
      }
    },
    *deleteImage({ payload, states }, { call, put }) {
      const response = yield call(deleteResource, payload);
      if (response && response.success) {
        message.success('删除图片成功');
        yield put({
          type: 'fetchImageList',
          payload: states,
        });
      }
    },
    *deleteVideo({ payload, states }, { call, put }) {
      const response = yield call(deleteResource, payload);
      if (response && response.success) {
        message.success('删除视频成功');
        yield put({
          type: 'fetchVideoList',
          payload: states,
        });
      }
    },
  },

  reducers: {
    querySuccess(state, action) {
      return {
        ...state,
        ...action.payload,
      };
    },
    clearState(state) {
      return {
        ...state,
        list: [],
        pageSize: 15,
        pageNo: 1,
      };
    },
  },
};
