import React, { Component } from 'react';
import pathToRegexp from 'path-to-regexp';
import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import { Form, Table, Modal, Card, Button, Input, Switch } from 'antd';
import { renderStatus, statusToBool, boolToStatus } from '../../../utils/utils';
import AXDragSortTable from '../../../components/AXDragSortTable';
import Selector from '../../../components/AXTableSelector/Selector';
import FooterToolbar from '../../../components/FooterToolbar';
import styles from './TagGroupCreate.less';


const formItemLayout = {
  labelCol: {
    xs: { span: 24 },
    sm: { span: 3 },
  },
  wrapperCol: {
    xs: { span: 24 },
    sm: { span: 14 },
    md: { span: 12 },
  },
};

@connect(({ loading, tagGroup, merchant }) => ({
  merchant,
  tagGroup,
  submitting: loading.models.tagGroup,
  mLoading: loading.models.merchant,
}))
@Form.create()
export default class TagCreatePage extends Component {
  state = {
    merchantSelectorDestroy: true,
  };
  componentWillMount() {
    const match = pathToRegexp('/frontend/tagGroup/create').exec(this.props.location.pathname);
    if (match) {
      this.cleanPageState();
    }
  }
  componentDidMount() {
    const matchId = this.isEdit();
    if (matchId) {
      this.props.dispatch({
        type: 'tagGroup/fetchTagGroupItem',
        payload: { groupId: matchId },
      });
    }
  }
  isEdit = () => {
    const { location } = this.props;
    const match = pathToRegexp('/frontend/tagGroup/edit/:id').exec(location.pathname);
    if (match) {
      return match[1];
    }
    return false;
  }
  cleanPageState=() => {
    this.props.dispatch({
      type: 'tagGroup/cleanItemState',
      payload: {},
    });
  }
  selectorDataFetcher=(name, params) => {
    switch (name) {
      case 'Merchant':
        this.props.dispatch({
          type: 'merchant/fetchMerchantList',
          payload: params,
        });
        break;
      case 'Tag':
        this.props.dispatch({
          type: 'tag/fetchTagList',
          payload: params,
        });
        break;
      default:
        break;
    }
  }
  handleMerchantSelectorModalShow = () => {
    this.setState({
      merchantSelectorDestroy: false,
    });
    this.selectorDataFetcher('Merchant');
  }
  handleMerchantSelectorChange = (params) => {
    this.selectorDataFetcher('Merchant', params);
  }
  handleMerchantSelectorCancel = () => {
    this.setState({
      merchantSelectorDestroy: true,
    });
  }
  handleMerchantSelectorFinish = (rows) => {
    this.setState({
      merchantSelectorDestroy: true,
    });
    if (!rows || !rows.length) {
      return;
    }
    const { id, name } = rows[0];
    this.props.dispatch({
      type: 'tagGroup/fixCurrentItem',
      payload: {
        merchantId: id,
        merchantName: name,
      },
    });
  }
  handleDragSortTableChange = (rows) => {
    this.props.dispatch({
      type: 'tagGroup/fixCurrentItem',
      payload: { tagList: rows },
    });
  }
  handlePageBack = () => {
    this.props.dispatch(routerRedux.push({
      pathname: '/frontend/tagGroup',
      state: this.props.location.state,
    }));
  }
  handlePageSubmit = (e) => {
    e.preventDefault();
    this.props.form.validateFieldsAndScroll((err, values) => {
      if (!err) {
        const { code, name, status } = values;
        const { tagGroup } = this.props;
        const { currentItem } = tagGroup;
        const { tagList = [], merchantId } = currentItem;
        const id = this.isEdit();
        if (id) {
          this.props.dispatch({
            type: 'tagGroup/updateTagGroupItem',
            payload: {
              id,
              code,
              name,
              merchantId,
              status: boolToStatus(status),
              tagList: tagList.map(item => item.id),
            },
            states: this.props.location.state,
          });
        } else {
          this.props.dispatch({
            type: 'tagGroup/createTagGroupItem',
            payload: {
              code,
              name,
              merchantId,
              status: boolToStatus(status),
              tagList: tagList.map(item => item.id),
            },
            states: this.props.location.state,
          });
        }
      }
    });
  }

  render() {
    const { form, submitting, mLoading, merchant, tagGroup } = this.props;
    const { merchantSelectorDestroy } = this.state;
    const { currentItem } = tagGroup;
    const { status, code, name, merchantName, tagList = [] } = currentItem;
    const { getFieldDecorator } = form;

    const merchantData = [{
      fieldName: '渠道名称',
      value: merchantName,
      key: 'merchantName',
    }];
    const merchantColumns = [{
      dataIndex: 'fieldName',
      key: 1,
      width: '30%',
    }, {
      dataIndex: 'value',
      key: 2,
      width: '70%',
    }];
    const tagColumns = [{
      title: '标签名称',
      dataIndex: 'name',
      key: 1,
      width: '35%',
    }, {
      title: '标签状态',
      dataIndex: 'status',
      render: text => renderStatus(text),
      key: 2,
    }];
    const getMerchantModal = () => {
      return (
        <Modal
          width={1100}
          footer={null}
          visible
          title="厂商列表"
          maskClosable={false}
          onCancel={this.handleMerchantSelectorCancel}
        >
          <Selector
            multiple={false}
            loading={mLoading}
            selectorName="Merchant"
            list={merchant.list}
            pageNo={merchant.pageNo}
            pageSize={merchant.pageSize}
            totalSize={merchant.totalSize}
            onCancel={this.handleMerchantSelectorCancel}
            onChange={this.handleMerchantSelectorChange}
            onFinish={this.handleMerchantSelectorFinish}
          />
        </Modal>
      );
    };
    const renderMerchantCardName = () => {
      return (
        <div className={styles.cardName}>
          <span>
            <a disabled={!!this.isEdit()} onClick={this.handleMerchantSelectorModalShow}>所属渠道</a>
          </span>
        </div>
      );
    };
    return (
      <div>
        {/* 基础信息Card */}
        <Card title="基础信息" style={{ marginBottom: 16 }}>
          <Form>
            <Form.Item hasFeedback label="标签组编号" {...formItemLayout}>
              {getFieldDecorator('code', {
                rules: [
                  {
                    required: true, message: '请填写标签组编号',
                  }, {
                    pattern: /^[a-zA-Z0-9|_|-]+$/g, message: '编号包含非法字符',
                  },
                ],
                initialValue: code,
              })(
                <Input
                  placeholder="请输入"
                  disabled={!!this.isEdit()}
                />
              )}
            </Form.Item>
            <Form.Item hasFeedback label="标签组名称" {...formItemLayout}>
              {getFieldDecorator('name', {
                rules: [{ required: true, message: '请填写标签名称' }],
                initialValue: name,
              })(
                <Input placeholder="请输入" />
              )}
            </Form.Item>
            <Form.Item label="标签组状态" {...formItemLayout}>
              {getFieldDecorator('status', {
                valuePropName: 'checked',
                initialValue: statusToBool(status),
              })(
                <Switch checkedChildren="正常" unCheckedChildren="删除" />
              )}
            </Form.Item>
          </Form>
        </Card>
        {/* 厂商选择Card */}
        <Card title={renderMerchantCardName()} style={{ marginBottom: 16 }}>
          <Table
            bordered
            showHeader={false}
            pagination={false}
            columns={merchantColumns}
            dataSource={merchantData}
          />
          {!merchantSelectorDestroy && getMerchantModal()}
        </Card>
        {/* 标签排序Card */}
        {this.isEdit() &&
          <Card title="标签列表" style={{marginBottom: 70}}>
            <AXDragSortTable
              data={tagList}
              columns={tagColumns}
              onChange={this.handleDragSortTableChange}
            />
          </Card>
        }
        <FooterToolbar style={{ width: '100%' }}>
          <Button
            onClick={this.handlePageBack}
            style={{ marginRight: 10 }}
          >取消
          </Button>
          <Button
            type="primary"
            loading={submitting}
            onClick={this.handlePageSubmit}
          >提交
          </Button>
        </FooterToolbar>
      </div>
    );
  }
}
