import React, { Component } from 'react';
import moment from 'moment';
import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import { Card, Modal, Form, Button, message } from 'antd';
import { StandardTableList } from '../../../components/AXList/index';
import AXRemoteSelect from '../../../components/AXRemoteSelect/index';
import Ellipsis from '../../../components/Ellipsis/index';
import { addRowKey, renderStatus, renderBindStatus } from '../../../utils/utils';
import { Hotax } from '../../../utils/config';

const Message = message;

const formItemLayout = {
  labelCol: {
    xs: { span: 24 },
    sm: { span: 7 },
  },
  wrapperCol: {
    xs: { span: 24 },
    sm: { span: 15 },
    md: { span: 13 },
  },
};
function arrayDataFormatter(data) {
  return data.map((item) => {
    return {
      text: item.name,
      value: item.id,
    };
  });
}

@Form.create()
@connect(({ loading, campus, merchant, terminal }) => ({
  campus,
  merchant,
  terminal,
  fetching1: loading.models.merchant,
  fetching2: loading.models.campus,
  loading: loading.models.terminal,
}))
export default class TerminalListPage extends Component {
  constructor(props) {
    super(props);
    const { state } = props.location;
    this.state = {
      UIParams: (state || {}).UIParams, // 组件的状态参数
      Queryers: (state || {}).Queryers, // 查询的条件参数
      filterModalDestroy: true,
    };
  }
  componentDidMount() {
    this.props.dispatch({
      type: 'terminal/fetchTerminalList',
      payload: { ...this.state.Queryers },
    });
  }
  handleCreateOperation = () => {
    this.props.dispatch(routerRedux.push({
      pathname: '/terminal/user/create',
      state: this.state,
    }));
  }
  handleDeviceUnboundOperation = (item) => {
    Modal.confirm({
      okText: '确定',
      cancelText: '取消',
      title: '你确定要解绑该账户吗?',
      content: '解绑后将导致已登录该账号的终端设备登录失效',
      onOk: () => {
        this.props.dispatch({
          type: 'terminal/deviceUnbound',
          payload: { id: item.id },
          states: this.state,
        });
      },
    });
  }
  handleDisableOperation = (item) => {
    Modal.confirm({
      okText: '禁用',
      cancelText: '取消',
      title: '你确定要禁用该终端账户吗?',
      content: '禁用该终端账户后已登录的终端会失效且不能登录',
      onOk: () => {
        this.props.dispatch({
          type: 'terminal/deleteTerminalItem',
          payload: { id: item.id },
          states: this.state,
        });
      },
    });
  }
  handleRecoverOperation = (item) => {
    Modal.confirm({
      okText: '确定',
      cancelText: '取消',
      title: '你确定要解除对该终端账号的禁用状态吗?',
      onOk: () => {
        this.props.dispatch({
          type: 'terminal/recoverTerminalItem',
          payload: {
            id: item.id,
            status: Hotax.STATUS_NORMAL,
          },
          states: this.state,
        });
      },
    });
  }
  handleEditOperation = (item) => {
    this.props.dispatch(routerRedux.push({
      pathname: `/terminal/user/edit/${item.id}`,
      state: {
        currentItem: item,
        ...this.state,
      },
    }));
  }
  handleFilterOperation = (params, states) => {
    this.props.dispatch({
      type: 'terminal/fetchTerminalList',
      payload: params,
    });
    this.setState({
      UIParams: states,
      Queryers: params,
    });
  }
  handleModalFilterOperation = () => {
    const { getFieldsValue } = this.props.form;
    const { merchantIds, campusIds } = getFieldsValue();
    this.props.dispatch({
      type: 'terminal/fetchTerminalList',
      payload: {
        ...this.state.Queryers,
        merchantId: merchantIds[0],
        campusId: campusIds[0],
      },
    });
    this.handleFilterModalDestroy();
  }
  handleBatchOperation = () => {
    Message.info('暂不支持批量操作!');
  }
  handleFilterModalShow = () => {
    this.setState({ filterModalDestroy: false });
  }
  handleFilterModalDestroy = () => {
    this.setState({ filterModalDestroy: true });
  }
  handleMerchantRemoteSelectSearch = (value) => {
    this.props.dispatch({
      type: 'merchant/fetchMerchantList',
      payload: {
        pageSize: 50,
        name: value,
      },
    });
  }
  handleCampusRemoteSelectSearch = (value) => {
    this.props.dispatch({
      type: 'campus/fetchCampusList',
      payload: {
        pageSize: 50,
        name: value,
      },
    });
  }

  render() {
    const { loading, fetching1, fetching2, form, campus, merchant, terminal } = this.props;
    const { list, totalSize, pageSize, pageNo } = terminal;
    const { getFieldDecorator } = form;

    const renderCampusName = (name) => {
      return (
        <Ellipsis tooltip lines={1}>{name}</Ellipsis>
      );
    };
    const renderOperation = (item) => {
      return (
        <div>
          <Button
            size="small"
            className="editBtn"
            onClick={() => this.handleEditOperation(item)}
          >编辑
          </Button>
          <Button
            size="small"
            className="depositBtn"
            onClick={() => this.handleDeviceUnboundOperation(item)}
            disabled={!item.deviceStatus}
          >解绑
          </Button>
          {
            item.status === Hotax.STATUS_NORMAL ? (
              <Button
                size="small"
                className="delBtn"
                onClick={() => this.handleDisableOperation(item)}
              >禁用
              </Button>
            ) : (
              <Button
                size="small"
                className="delBtn"
                onClick={() => this.handleRecoverOperation(item)}
              >解禁
              </Button>
            )
          }
        </div>
      );
    };
    const batchActions = [{
      key: 'delete',
      name: '批量禁用',
    }, {
      key: 'recovery',
      name: '批量解禁',
    }, {
      key: 'unbound',
      name: '批量解绑',
    }];
    const basicSearch = {
      keys: [{
        name: '终端编号',
        field: 'code',
      }, {
        name: '终端名称',
        field: 'name',
      }],
    };
    const pagination = {
      pageNo,
      pageSize,
      totalSize,
    };
    const columns = [{
      title: '终端编号',
      key: 1,
      dataIndex: 'code',
      width: '15%',
    }, {
      title: '终端名称',
      key: 2,
      dataIndex: 'name',
      width: '12%',
    }, {
      title: '所属校区',
      key: 3,
      dataIndex: 'campusName',
      render: text => renderCampusName(text),
      width: '13%',
    }, {
      title: '所属渠道',
      key: 4,
      dataIndex: 'merchantName',
      width: '10%',
    }, {
      title: '账号状态',
      key: 5,
      dataIndex: 'status',
      render: text => renderStatus(text, '已禁用'),
      width: '8%',
    }, {
      title: '绑定状态',
      key: 6,
      dataIndex: 'deviceStatus',
      render: text => renderBindStatus(text),
      width: '9%',
    }, {
      title: '更新时间',
      key: 7,
      dataIndex: 'gmtModified',
      render: text => moment(text).format('YYYY-MM-DD HH:mm:ss'),
      width: '15%',
    }, {
      title: '操作',
      key: 8,
      dataIndex: 'operation',
      render: (_, record) => renderOperation(record),
      width: '18%',
      align: 'right',
    }];
    return (
      <Card>
        <StandardTableList
          columns={columns}
          loading={loading}
          dataSource={addRowKey(list)}
          header={{
            basicSearch,
            onAdvanceFilterClick: this.handleFilterModalShow,
            onFilterClick: this.handleFilterOperation,
            onCreateClick: this.handleCreateOperation,
          }}
          footer={{
            pagination,
            batchActions,
            onBatchClick: this.handleBatchOperation,
          }}
          keepUIState={{ ...this.state.UIParams }}
        />
        {!this.state.filterModalDestroy && (
          <Modal
            width={600}
            visible
            title="高级筛选"
            okText="筛选"
            cancelText="取消"
            maskClosable={false}
            onCancel={this.handleFilterModalDestroy}
            onOk={this.handleModalFilterOperation}
          >
            <Form>
              <Form.Item label="所属商户" {...formItemLayout}>
                {getFieldDecorator('merchantIds', {
                    initialValue: [],
                  })(
                    <AXRemoteSelect
                      fetching={fetching1}
                      dataSource={arrayDataFormatter(merchant.list)}
                      onSearch={this.handleMerchantRemoteSelectSearch}
                    />
                  )}
              </Form.Item>
              <Form.Item label="所属校区" {...formItemLayout}>
                {getFieldDecorator('campusIds', {
                    initialValue: [],
                  })(
                    <AXRemoteSelect
                      fetching={fetching2}
                      dataSource={arrayDataFormatter(campus.list)}
                      onSearch={this.handleCampusRemoteSelectSearch}
                    />
                  )}
              </Form.Item>
            </Form>
          </Modal>
        )}
      </Card>
    );
  }
}
