import React, { Component } from 'react';
import moment from 'moment';
import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import { Card, Modal, Button, message } from 'antd';
import { StandardTableList } from '../../../components/AXList';
import { addRowKey, renderStatus } from '../../../utils/utils';
import { Hotax } from '../../../utils/config';

const Message = message;

@connect(({ loading, terminal }) => ({
  terminal,
  loading: loading.models.terminal,
}))
export default class WhiteListPage extends Component {
  constructor(props) {
    super(props);
    const { state } = props.location;
    this.state = {
      UIParams: (state || {}).UIParams, // 组件的状态参数
      Queryers: (state || {}).Queryers, // 查询的条件参数
    };
  }
  componentDidMount() {
    this.props.dispatch({
      type: 'terminal/fetchSpecialTerminalList',
      payload: { ...this.state.Queryers },
    });
  }
  handleCreateOperation = () => {
    this.props.dispatch(routerRedux.push({
      pathname: '/terminal/whitelist/create',
      state: this.state,
    }));
  }
  handleDeleteOperation = (item) => {
    Modal.confirm({
      okText: '确定',
      cancelText: '取消',
      title: '你确定要删除该白名单用户吗?',
      content: '删除后该终端账号将不再具有所有课程权限',
      onOk: () => {
        this.props.dispatch({
          type: 'terminal/deleteSpecialTerminalItem',
          payload: { userId: item.userId },
          states: this.state,
        });
      },
    });
  }
  handleEditOperation = (item) => {
    this.props.dispatch(routerRedux.push({
      pathname: `/terminal/whitelist/edit/${item.userId}`,
      state: {
        currentItem: item,
        ...this.state,
      },
    }));
  }
  handleFilterOperation = (params, states) => {
    this.props.dispatch({
      type: 'terminal/fetchSpecialTerminalList',
      payload: params,
    });
    this.setState({
      UIParams: states,
      Queryers: params,
    });
  }
  handleBatchOperation = () => {
    Message.info('暂不支持批量操作!');
  }

  render() {
    const { loading, terminal } = this.props;
    const { list, totalSize, pageSize, pageNo } = terminal;

    const renderOperation = (item) => {
      return (
        <div>
          <Button
            size="small"
            disabled={item.status === Hotax.STATUS_DELETE}
            className="editBtn"
            onClick={() => this.handleEditOperation(item)}
          >
            编辑
          </Button>
          <Button
            size="small"
            className="delBtn"
            onClick={() => this.handleDeleteOperation(item)}
          >
            删除
          </Button>
        </div>
      );
    };
    const batchActions = [{
      key: 'delete',
      name: '批量删除',
    }];
    const basicSearch = {
      keys: [{
        name: '终端编号',
        field: 'code',
      }],
    };
    const pagination = {
      pageNo,
      pageSize,
      totalSize,
    };
    const columns = [{
      title: '终端编号',
      key: 1,
      dataIndex: 'code',
      width: '15%',
    }, {
      title: '所属渠道',
      key: 2,
      dataIndex: 'merchantName',
      width: '13%',
    }, {
      title: '起始日期',
      key: 3,
      dataIndex: 'startTime',
      render: text => moment(text).format('YYYY-MM-DD HH:mm:ss'),
      width: '17%',
    }, {
      title: '截止日期',
      key: 4,
      dataIndex: 'endTime',
      render: (text) => {
        if (text) {
          return moment(text).format('YYYY-MM-DD HH:mm:ss');
        } else {
          return '永久';
        }
      },
      width: '17%',
    }, {
      title: '状态',
      key: 5,
      dataIndex: 'status',
      render: text => renderStatus(text),
      width: '10%',
    }, {
      title: '更新时间',
      key: 6,
      dataIndex: 'gmtModified',
      render: text => moment(text).format('YYYY-MM-DD HH:mm:ss'),
      width: '15%',
    }, {
      title: '操作',
      key: 7,
      dataIndex: 'operation',
      render: (_, record) => renderOperation(record),
      width: '13%',
      align: 'right',
    }];
    return (
      <Card>
        <StandardTableList
          columns={columns}
          loading={loading}
          dataSource={addRowKey(list)}
          header={{
            basicSearch,
            onFilterClick: this.handleFilterOperation,
            onCreateClick: this.handleCreateOperation,
          }}
          footer={{
            pagination,
            batchActions,
            onBatchClick: this.handleBatchOperation,
          }}
          keepUIState={{ ...this.state.UIParams }}
        />
      </Card>
    );
  }
}
