/* eslint-disable no-trailing-spaces */
import React, { Component } from 'react';
import pathToRegexp from 'path-to-regexp';
import { connect } from 'dva';
import { Card, Form, Table, List, Steps, Select, Button, Input, InputNumber, Icon } from 'antd';
import FooterToolbar from '../../../components/FooterToolbar';
import { checkProductType, toDecimal2 } from '../../../utils/utils';
import { Hotax } from '../../../utils/config';
import styles from './ShopCartDetail.less';

const formItemLayout = {
  labelCol: {
    xs: { span: 24 },
    sm: { span: 7 },
  },
  wrapperCol: {
    xs: { span: 24 },
    sm: { span: 14 },
    md: { span: 10 },
  },
};

@connect(({ trade }) => ({
  trade,
}))
@Form.create()
export default class ShopCartDetail extends Component {
  state = {
    currentStep: 0,
    selectedRowKeys: [],
    adjustPrice: 0,
    deliveryInfo: {},
    width: '100%',
  };
  componentDidMount() {
    // 监听视图窗口变化
    window.addEventListener('resize', this.resizeFooterToolbar);
    this.resizeFooterToolbar();
    // 匹配路径中的userId
    const match = pathToRegexp('/trade/shopcart/detail/:id')
      .exec(this.props.location.pathname);
    if (match) {
      this.props.dispatch({
        type: 'trade/fetchTerminalBuyMsg',
        payload: { userId: match[1] },
      });
    }
    // 获取收货信息
    const { deliveryInfo } = this.props.location.state;
    this.setState({ deliveryInfo });
  }
  componentWillUnmount() {
    window.removeEventListener('resize', this.resizeFooterToolbar);
  }
  resizeFooterToolbar = () => {
    const sider = document.querySelectorAll('.ant-layout-sider')[0];
    const width = `calc(100% - ${sider.style.width})`;
    if (this.state.width !== width) {
      this.setState({ width });
    }
  };
  // 响应多选变化
  handleRowSelectChange = (selectedRowKeys) => {
    this.setState({ selectedRowKeys });
  };
  // 响应价格变化
  handleGoodsSelectChange = (rowIndex, goodsId) => {
    const { trade } = this.props;
    const { list } = trade;
    const targetRow = list[rowIndex];
    const { goods } = targetRow;
    const targetGoods = goods.find(goodsItem => goodsItem.id === goodsId);
    if (targetGoods) {
      targetRow.selectedGoodsId = targetGoods.id;
      targetRow.selectedChargeUnit = targetGoods.chargeUnit;
      targetRow.selectedPrice = targetGoods.merchantPrice;
      list[rowIndex] = targetRow;
      this.props.dispatch({
        type: 'trade/fixList',
        payload: { list },
      });
    }
  };
  // 响应数量变化 - 填写
  handleQuantityInputChange = (rowIndex, e) => {
    const { trade } = this.props;
    const { list } = trade;
    const targetRow = list[rowIndex];
    targetRow.quantity = e.target.value;
    list[rowIndex] = targetRow;
    this.props.dispatch({
      type: 'trade/fixList',
      payload: { list },
    });
  };
  // 响应数量变化 - 加减
  handleQuantityStepChange = (rowIndex, type) => {
    const { trade } = this.props;
    const { list } = trade;
    const targetRow = list[rowIndex];
    const { quantity } = targetRow;
    if (type === 'plus') {
      targetRow.quantity = quantity + 1;
    } else if (type === 'minus' && quantity > 1) {
      targetRow.quantity = quantity - 1;
    }
    list[rowIndex] = targetRow;
    this.props.dispatch({
      type: 'trade/fixList',
      payload: { list },
    });
  };
  // 优惠价格调整
  handleAdjustPriceChange = (value) => {
    this.setState({ adjustPrice: value });
  };
  // 下一步
  nextStep = () => {
    const { currentStep } = this.state;
    if (currentStep === 1) {
      this.props.form.validateFieldsAndScroll((error, values) => {
        if (!error) {
          this.setState({
            currentStep: this.state.currentStep + 1,
            deliveryInfo: { ...values },
          });
        }
      });
      return;
    }
    this.setState({ currentStep: this.state.currentStep + 1 });
  };
  // 上一步
  prevStep = () => {
    this.setState({ currentStep: this.state.currentStep - 1 });
  };
  // 提交订单
  handleSubmitOrder = (goodsList) => {
    const { deliveryInfo, adjustPrice } = this.state;
    const { contactName, mobile, address } = deliveryInfo;
    const goods = goodsList.map(item => ({
      goodsId: item.selectedGoodsId,
      quantity: item.quantity,
    }));
    const uid = pathToRegexp('/trade/shopcart/detail/:id')
      .exec(this.props.location.pathname)[1];
    const submitBody = {
      uid,
      mobile,
      address,
      goods,
      adjustPrice,
      isFromCart: true,
      name: contactName,
      status: Hotax.ORDER_UNPAID,
    };
    this.props.dispatch({
      type: 'trade/createOrderItem',
      payload: submitBody,
    });
  };

  render() {
    const { currentStep, selectedRowKeys, adjustPrice = 0, deliveryInfo } = this.state;
    const { trade, form } = this.props;
    const { getFieldDecorator } = form;
    const { contactName, mobile, address } = deliveryInfo;
    const { list } = trade;

    // 找出选中的商品价格
    const findSelectedPrice = (goodsArr) => {
      if (!goodsArr) {
        return;
      }
      let selectedGoodsId = null;
      let selectedChargeUnit = null;
      let selectedPrice = null;
      goodsArr.forEach((item) => {
        if (item.isInCart) {
          selectedGoodsId = item.id;
          selectedChargeUnit = item.chargeUnit;
          selectedPrice = parseInt(item.merchantPrice, 10);
        }
      });
      return { selectedGoodsId, selectedChargeUnit, selectedPrice };
    };
    // 处理每条数据
    const rowDataFormatter = (rows) => {
      const newRows = [];
      for (let row of rows) {
        // 默认数量
        if (!row.quantity) {
          row.quantity = 1;
        }
        // 默认选中价格
        if (!row.selectedGoodsId) {
          const price = findSelectedPrice(row.goods);
          row = { ...row, ...price };
        }
        // 小计
        row.subTotal = row.quantity * row.selectedPrice;
        newRows.push(row);
      }
      return newRows;
    };

    // 商品清单表格列配置
    const goodsColumns = [{
      title: '商品编号',
      dataIndex: 'code',
      key: 0,
      render: text => (
        <a>{text}</a>
      ),
      width: '16%',
    }, {
      title: '商品名称',
      dataIndex: 'name',
      key: 1,
      render: text => (
        <a>{text}</a>
      ),
      width: '24%',
    }, {
      title: '商品类型',
      key: 2,
      dataIndex: 'type',
      render: text => checkProductType(text),
      width: '15%',
      align: 'center',
    }, {
      title: '单价',
      key: 3,
      dataIndex: 'goods',
      render: (goodsArr, record, index) => {
        if (!goodsArr) {
          return null;
        }
        return (
          <Select
            style={{ width: 150 }}
            value={record.selectedGoodsId}
            onChange={goodsId => this.handleGoodsSelectChange(index, goodsId)}
          >
            {
              goodsArr.map(item => (
                <Select.Option key={item.id} value={item.id}>
                  {`¥${item.merchantPrice}/${item.chargeUnit}`}
                </Select.Option>
              ))
            }
          </Select>
        );
      },
      width: '12%',
      align: 'center',
    }, {
      title: '数量',
      key: 4,
      dataIndex: 'quantity',
      width: '15%',
      render: (text, _, index) => {
        return (
          <Input
            style={{ width: 150 }}
            addonBefore={
              <Icon
                type="minus"
                className={styles.icon}
                onClick={() => this.handleQuantityStepChange(index, 'minus')}
              />
            }
            addonAfter={
              <Icon
                type="plus"
                className={styles.icon}
                onClick={() => this.handleQuantityStepChange(index, 'plus')}
              />
            }
            value={text}
            onChange={e => this.handleQuantityInputChange(index, e)}
          />
        );
      },
      align: 'center',
    }, {
      title: '小计',
      key: 5,
      dataIndex: 'subTotal',
      width: '13%',
      render: text => (
        <span style={{ fontWeight: 500 }}>{`¥${text}`}</span>
      ),
      align: 'center',
    }];

    const rowSelection = {
      selectedRowKeys,
      onChange: this.handleRowSelectChange,
    };

    // 按需求处理后的数据
    const formattedDataSource = rowDataFormatter(list);

    // 计算选中的行
    const selectedRows = formattedDataSource.filter(
      item => selectedRowKeys.indexOf(item.id) !== -1
    );

    // 计算总价
    const computeTotalPrice = (rows) => {
      let sum = 0;
      rows.forEach((row) => {
        sum += row.subTotal;
      });
      return sum;
    };
    const totalPrice = computeTotalPrice(selectedRows);

    // 商品选择Card内容
    const goodsSelectCard = () => {
      return (
        <Card title="商品清单" style={{ marginTop: 10 }}>
          <Table
            border={false}
            pagination={false}
            columns={goodsColumns}
            dataSource={formattedDataSource}
            rowSelection={rowSelection}
            rowKey={record => record.id}
          />
        </Card>
      );
    };

    // 收货信息Card内容
    const addressInfoCard = () => {
      return (
        <Card title="收货信息" style={{ marginTop: 10 }}>
          <Form>
            <Form.Item label="收货人" {...formItemLayout}>
              {getFieldDecorator('contactName', {
                rules: [{ required: true, message: '请填写收货人' }],
                initialValue: contactName,
              })(
                <Input placeholder="请填写" />
              )}
            </Form.Item>
            <Form.Item label="收货地址" {...formItemLayout}>
              {getFieldDecorator('address', {
                rules: [{ required: true, message: '请填写收货地址' }],
                initialValue: address,
              })(
                <Input placeholder="请填写" />
              )}
            </Form.Item>
            <Form.Item label="手机号码" {...formItemLayout}>
              {getFieldDecorator('mobile', {
                rules: [
                  {
                    required: true, message: '请填写联系电话',
                  }, {
                    pattern: /^[1][34578][0-9]{9}$/g, message: '请输入11位有效手机号!',
                  },
                ],
                initialValue: mobile,
              })(
                <Input placeholder="请填写" />
              )}
            </Form.Item>
          </Form>
        </Card>
      );
    };

    // 确认订单Card内容
    const confirmInfoCard = () => {
      const columns = [{
        title: '商品名称',
        dataIndex: 'name',
        key: 1,
        width: '20%',
      }, {
        title: '商品编号',
        dataIndex: 'code',
        key: 2,
        width: '20%',
      }, {
        title: '商品类型',
        dataIndex: 'type',
        key: 3,
        align: 'center',
        render: text => checkProductType(text),
        width: '12%',
      }, {
        title: '单价',
        dataIndex: 'selectedPrice',
        key: 4,
        align: 'center',
        render: text => `¥${text}`,
        width: '12%',
      }, {
        title: '数量',
        dataIndex: 'quantity',
        key: 5,
        align: 'center',
        render: text => `×${text}`,
        width: '12%',
      }, {
        title: '单位',
        dataIndex: 'selectedChargeUnit',
        key: 6,
        align: 'center',
        width: '12%',
      }, {
        title: '小计',
        dataIndex: 'subTotal',
        key: 7,
        align: 'center',
        render: text => `¥${text}`,
        width: '12%',
      }];
      return (
        <div>
          <Card title="商品清单" style={{ marginTop: 10 }}>
            <Table
              bordered
              columns={columns}
              pagination={false}
              rowKey={record => record.id}
              dataSource={selectedRows}
            />
          </Card>
          <Card title="收货信息" style={{ marginTop: 10 }}>
            <List
              bordered
              size="small"
              dataSource={[
                `收货人: ${contactName || ''}`,
                `收货地址: ${address || ''}`,
                `手机号码: ${mobile || ''}`,
              ]}
              renderItem={item => <List.Item>{item}</List.Item>}
            />
          </Card>
        </div>
      );
    };

    // 步骤条配置
    const steps = [{
      title: '选择商品',
      stepKey: 0,
      icon: 'shop',
      content: goodsSelectCard(),
    }, {
      title: '填写收货信息',
      stepKey: 1,
      icon: 'environment',
      content: addressInfoCard(),
    }, {
      title: '确认订单',
      stepKey: 2,
      icon: 'profile',
      content: confirmInfoCard(),
    }];

    return (
      <div>
        <Steps current={currentStep}>
          {steps.map((item, index) =>
            (
              <Steps.Step
                key={item.stepKey}
                title={item.title}
                icon={<Icon type={steps[index].icon} />}
              />
            )
          )}
        </Steps>
        {steps[currentStep].content}
        <FooterToolbar
          extra={
            <div>
              <span className={styles.quantity}>
                已选择&nbsp;
                <span style={{ color: '#f60', fontWeight: 500 }}>
                  {selectedRows.length}
                </span>
                &nbsp;件商品
              </span>
              <span className={styles.adjustPrice}>
                <span style={{ color: '#2f7d0d' }}>
                  优惠:&nbsp;&nbsp;¥&nbsp;
                </span>
                <InputNumber
                  min={0}
                  max={totalPrice}
                  size="small"
                  value={adjustPrice}
                  onChange={this.handleAdjustPriceChange}
                />
              </span>
              <span className={styles.totalPrice}>
                总价:&nbsp;&nbsp;&nbsp;
                <span style={{ color: '#f60', fontSize: 24 }}>
                  {`¥ ${toDecimal2(totalPrice - adjustPrice)}`}
                </span>
              </span>
            </div>
          }
          style={{ width: this.state.width }}
        >
          {
            (currentStep > 0) && (
              <Button onClick={this.prevStep}>
                {`上一步: ${steps[currentStep - 1].title}`}
              </Button>
            )
          }
          {
            (currentStep < 2) ? (
              <Button
                type="primary"
                disabled={selectedRowKeys.length === 0}
                onClick={this.nextStep}
              >
                {`下一步: ${steps[currentStep + 1].title}`}
              </Button>
            ) : (
              <Button
                type="primary"
                onClick={() => this.handleSubmitOrder(selectedRows)}
              >
                确认下单
              </Button>
            )
          }
        </FooterToolbar>
      </div>
    );
  }
}
