import { stringify } from 'qs';
import request from '../utils/request';
import { api, Hotax } from '../utils/config';

export async function queryMerchantList(params) {
  const newParams = {
    pageSize: Hotax.PAGE_SIZE,
    ...params,
  };
  return request(`${api.merchant}?${stringify(newParams)}`);
}

export async function queryMerchantItem({ id }) {
  return request(`${api.merchantItem}/${id}`);
}

export async function createMerchantItem(params) {
  const options = {
    method: 'POST',
    body: params,
  };
  return request(`${api.merchantItem}`, options);
}

export async function updateMerchantItem(params) {
  const options = {
    method: 'PUT',
    body: params,
  };
  return request(`${api.merchantItem}`, options);
}

export async function deleteMerchantItem({ id }) {
  const options = {
    method: 'DELETE',
  };
  return request(`${api.merchantItem}/${id}`, options);
}

export async function depositMerchantItem(params) {
  const options = {
    method: 'POST',
    body: params,
  };
  return request(`${api.deposit}`, options);
}

export async function queryMerchantRecommend({ merchantId }) {
  return request(`${api.recommend}/${merchantId}`);
}

export async function updateMerchantRecommend({ merchantId, idList }) {
  const options = {
    method: 'PUT',
    body: idList,
  };
  return request(`${api.recommend}/${merchantId}`, options);
}

export async function queryMerchantPoster({ merchantId }) {
  return request(`${api.poster}/${merchantId}`);
}

export async function queryMerchantPosterItem({ posterId }) {
  return request(`${api.posterItem}/${posterId}`);
}

export async function createMerchantPosterItem(params) {
  const options = {
    method: 'POST',
    body: params,
  };
  return request(`${api.posterItem}`, options);
}

export async function updateMerchantPosterItem(params) {
  const options = {
    method: 'PUT',
    body: params,
  };
  return request(`${api.posterItem}`, options);
}

export async function deleteMerchantPosterItem({ posterId }) {
  const options = {
    method: 'DELETE',
  };
  return request(`${api.posterItem}/${posterId}`, options);
}
